/*
	file automatically generated by make_test_files.pl
	Mon Jan  9 14:11:36 2006
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for MCORR
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_mcorr_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/



/* prototypes */

#define COMPARE_SAMPLE_SIZE   (10000)
#define VIOLATE_SAMPLE_SIZE   (20)

#include<../src/utils/matrix_source.h>
#include<../src/utils/vector_source.h>




/*---------------------------------------------------------------------------*/

#ifndef CHI2_FAILURES_TOLERATED
#  define CHI2_FAILURES_TOLERATED DEFAULT_CHI2_FAILURES_TOLERATED
#endif

/*---------------------------------------------------------------------------*/
/* [verbatim] */



int
mcorr_eigenvalues_test(int dim) 
/* returns 0 if everything is OK, 1 otherwise */
{
  UNUR_DISTR *corr_distr;
  UNUR_PAR   *corr_par;
  UNUR_GEN   *corr_gen;

  int i;
  int err = 0;
  
  double *mean;
  double *corr;
  double *eigenvalues, *eigenvalues_set;
  double *eigenvectors;

  mean = _unur_vector_new(dim);
  corr = _unur_vector_new(dim*dim);
  eigenvalues_set = _unur_vector_new(dim);
  eigenvalues = _unur_vector_new(dim);
  eigenvectors = _unur_vector_new(dim*dim);

  /* setting eigenvalues to const*{1,2,...,dim} with sum=dim */	  
  for (i=0; i<dim; i++) eigenvalues_set[i]=2*(1+i)/(1.+dim);	  

  corr_distr=unur_distr_correlation(dim);
  corr_par=unur_mcorr_new(corr_distr);
  unur_mcorr_set_eigenvalues(corr_par, eigenvalues_set);

  corr_gen=unur_init(corr_par);
  unur_sample_matr(corr_gen, corr);

  /* calculating eigenvectors of generated correlation matric */
  _unur_matrix_eigensystem(dim, corr, eigenvalues, eigenvectors);

  fprintf(TESTLOG,"dim = %d ...\t", dim);

  for (i=0; i<dim; i++)
    if (!_unur_FP_equal(eigenvalues[i], eigenvalues_set[i]))
       err++;

  if (err) {
    fprintf(TESTLOG,"failed\n");
    for (i=0; i<dim; i++)
      fprintf(TESTLOG,"\t%g  \t(instead of %g)\n",eigenvalues[i],eigenvalues_set[i]);
  }	
  else {
    fprintf(TESTLOG,"ok\n");
  }

  free(mean); 
  free(corr);
  free(eigenvectors);
  free(eigenvalues);
  free(eigenvalues_set);
  
  unur_distr_free(corr_distr);
  unur_free(corr_gen);

  return err;
}

/*---------------------------------------------------------------------------*/
/* [new] */

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_DISTR *distr = NULL;
   distr = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,29,(unur_mcorr_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,29,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* invalid distribution type */
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,35,(unur_mcorr_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,35,UNUR_ERR_DISTR_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/
/* [set] */

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_PAR   *par = NULL;
   double *eigenvalues = NULL;
   par = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,47,(unur_mcorr_set_eigenvalues( par, eigenvalues )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,47,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
}

{ /* invalid NULL ptr */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double *eigenvalues = NULL;
   distr = unur_distr_correlation(4); 
   par = unur_mcorr_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,56,(unur_mcorr_set_eigenvalues( par, eigenvalues )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,56,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
unur_par_free(par);
unur_distr_free(distr);
}

{ /* invalid parameter object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double eigenvalues[] = {1,2,3};
   distr = unur_distr_normal(NULL,0);
   par = unur_arou_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,67,(unur_mcorr_set_eigenvalues( par, eigenvalues )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,67,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;
unur_par_free(par);
unur_distr_free(distr);
}

{ /* invalid parameters */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double eigenvalues[] = {1,-2,3};
   distr = unur_distr_correlation(4); 
   par = unur_mcorr_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,78,(unur_mcorr_set_eigenvalues( par, eigenvalues )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,78,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;
unur_par_free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/
/* [get] */

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/
/* [chg] */

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/
/* [init] */

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_PAR   *par = NULL;
   par = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,99,(unur_init( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,99,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/
/* [reinit] */

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/
/* [sample] */

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* compare clone */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   distr = unur_distr_correlation(4); 
   par = NULL;
   gen = NULL; 


unur_errno = 0;
/* original generator object */
par = unur_mcorr_new(distr);
gen = unur_init(par);
n_tests_failed += (compare_matr_sequence_gen_start(TESTLOG,119,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += (compare_matr_sequence_gen(TESTLOG,125,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* compare clone */
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
UNUR_DISTR *distr = NULL;
   UNUR_GEN *clone;
   double eigenvalues[] = {1.,2.,3.,4.};
   par = NULL;
   gen = NULL;
   distr = unur_distr_correlation(4); 


unur_errno = 0;
/* original generator object */
par = unur_mcorr_new(distr);
unur_mcorr_set_eigenvalues( par, eigenvalues );
gen = unur_init(par);
n_tests_failed += (compare_matr_sequence_gen_start(TESTLOG,140,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += (compare_matr_sequence_gen(TESTLOG,146,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/
/* [validate] */

/*---------------------------------------------------------------------------*/

/* [validate] */

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	int rcode;
	double *darray;
	double fpm[10];

	rcode = 0;

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 0 */

/* free distributions */
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>CHI2_FAILURES_TOLERATED) ? 0 : 1;
	/* we accept CHI2_FAILURES_TOLERATED failures */
	(n_tests_failed>CHI2_FAILURES_TOLERATED) ? printf(" ==> failed] ") : printf(" ==> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/
/* [special] */

void test_special (void)
{
	/* set boolean to FALSE */
	int FAILED = 0;
  



  int errorsum = 0;
  int dim;
  
/* start test */
printf("[special "); fflush(stdout);
fprintf(TESTLOG,"\n[special]\n");

  /* test for eigenvalues */
  printf("\ntest eigenvalues"); 
  fprintf(TESTLOG,"\nTest eigenvalues:\n"); 

  for (dim=2; dim<10; dim++) {
    if (mcorr_eigenvalues_test(dim)) errorsum++;
  }

  /* test finished */
  FAILED = (errorsum==0) ? 0: 1;
    


	/* test finished */
	test_ok &= (FAILED) ? 0 : 1;
	(FAILED) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = special  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_special() */


/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_mcorr_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_mcorr_set_verify(UNUR_PAR *par, int verify) {return 0;}

/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_mcorr_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_mcorr_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(929669);
	unur_urng_fish_seed(929669);
	unur_urng_mstd_seed(929669);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_GENERIC
/*
	if (unur_urng_seed(NULL, 929669) != UNUR_SUCCESS) {
	  fprintf(stderr,"WARNING: Seed could not be set at random\n");
	}  
*/
#elif UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(929669)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {929669,929669,929669,929669,929669,929669};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("mcorr: ");

	/* run tests */
test_new();
test_set();
test_get();
test_chg();
test_init();
test_reinit();
test_sample();
test_validate();
test_special();


	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? EXIT_SUCCESS : EXIT_FAILURE );

} /* end of main */

